if(Ext.ux.tree.TreeGridSorter){
  Ext.apply(Ext.ux.tree.TreeGridSorter.prototype, {
    sortAscText  : "Сортировать по возрастанию",
    sortDescText : "Сортировать по убыванию"
  });
}

if(Ext.ux.tree.TreeGrid){
  Ext.apply(Ext.ux.tree.TreeGrid.prototype, {
    columnsText  : "Столбцы"
  });
}
